#!/usr/bin/env perl
#
#  chstructure.pl
#  2004-12-23 stvo 
#  $Id: chstructure.pl,v 1.5 2022/11/22 09:42:05 testx Exp testx $
#
use warnings;
use strict;

use constant Filetype => {
   F => 'Datei',
   D => 'Verzeichnis',
   L => 'Symbolischer Link',
   P => 'Pipe',
   S => 'Socket',
   C => 'character special',
   B => 'block special',
};

sub classify ($)
{
   my $fso = shift;
   lstat $fso;
   return 'F' if -f _;
   return 'D' if -d _;
   return 'L' if -l _;
   return 'P' if -p _;
   return 'S' if -S _;
   return 'C' if -c _;
   return 'B' if -b _;
   warn "fso <$fso> has unknown type";
   return;
}

sub traversedir (\%$$);
sub traversedir (\%$$)
{
   my $allobjs = shift;
   my $prefix = shift;
   my $dir = shift;

   opendir (DIR, $prefix . $dir) or do {
      warn "cannot open ", $prefix . $dir, "\n";
      return;
   };
   my @dirents = readdir DIR;
   closedir DIR;
   for my $d (sort @dirents) {
      next if ($d eq '.' or $d eq '..');
      my $ndir = $dir . '/' . $d;
      my $class = classify ($prefix . '/' . $ndir);
      next if ! defined $class;
      next if $class eq 'F';
      $allobjs->{$ndir} = $class;
      traversedir (%$allobjs, $prefix, $ndir) if $class eq 'D';
   }
}

sub usage ()
{
   print "chstructure.pl oldpath newpath\n";
}

sub main ()
{
   my $orig = $ARGV [0];
   my $new = $ARGV [1];

   my $rc = 0;

   return usage () if ! defined $orig || ! defined $new;

   traversedir (my %old, $orig, '');
   traversedir (my %allobjs, $new, '');

   for my $i (sort keys %allobjs) {
      if (defined $old{$i} and $old{$i} ne $allobjs{$i}) {
         printf "<%s>: alt <%s> zu importieren <%s>\n",
            $i, 
            Filetype->{$old{$i}},
            Filetype->{$allobjs{$i}};
         $rc |= 1;
      }
   }

   return $rc;
}

exit main ();
